'''info.py'''

from mediaMetadata import ffmpegFPS, vidTracks
from usefulFunctions import pipeToConsole

def getInfo(files, ffmpeg, ffprobe, log):
    for file in files:
        print(f'file: {file}')

        hasVid = pipeToConsole([ffprobe, '-v', 'error', '-show_streams',
            '-select_streams', 'v', file])
        hasAud = pipeToConsole([ffprobe, '-v', 'error', '-show_streams',
            '-select_streams', 'a', file])

        hasAud = len(hasAud) > 5
        hasVid = len(hasVid) > 5

        if(hasVid):
            fps = ffmpegFPS(ffmpeg, file, log)
            print(f' - fps: {fps}')

            res = pipeToConsole([ffprobe, '-v', 'error', '-select_streams', 'v:0',
                '-show_entries', 'stream=height,width', '-of', 'csv=s=x:p=0', file])
            print(f' - resolution: {res.strip()}')

            raw_data = pipeToConsole([ffprobe, '-v', 'error', '-select_streams',
                'v:0', '-show_entries', 'stream=codec_name,bit_rate', '-of',
                'compact=p=0:nk=1', file]).split('|')

            print(f' - video codec: {raw_data[0]}')
            if(raw_data[1].isnumeric()):
                vbit = str(int(int(raw_data[1]) / 1000)) + 'k'
            else:
                vbit = 'N/A'
            print(f' - video bitrate: {vbit}')

            if(hasAud):
                tracks = vidTracks(file, ffprobe, log)
                print(f' - audio tracks: {tracks}')

                for track in range(tracks):
                    print(f'   - Track #{track}')

                    raw_data = pipeToConsole([ffprobe, '-v', 'error', '-select_streams',
                        f'a:{track}', '-show_entries', 'stream=codec_name,sample_rate', '-of',
                        'compact=p=0:nk=1', file])

                    raw_data = raw_data.replace('\n', '').split('|')

                    acod = raw_data[0]
                    if(len(raw_data) > 1 and raw_data[1].isnumeric()):
                        sr = str(int(raw_data[1]) / 1000) + ' kHz'
                    else:
                        sr = 'N/A'

                    print(f'     - codec: {acod}')
                    print(f'     - samplerate: {sr}')

                    output = pipeToConsole([ffprobe, '-v', 'error', '-select_streams',
                        f'a:{track}', '-show_entries', 'stream=bit_rate', '-of',
                        'compact=p=0:nk=1', file]).strip()
                    if(output.isnumeric()):
                        abit = str(round(int(output) / 1000)) + 'k'
                        print(f'     - bitrate: {abit}')
            else:
                print(' - audio tracks: 0')
        elif(hasAud):
            raw_data = pipeToConsole([ffprobe, '-v', 'error', '-select_streams',
                'a:0', '-show_entries', 'stream=codec_name,sample_rate', '-of',
                'compact=p=0:nk=1', file]).split('|')

            acod = raw_data[0]
            sr = str(int(raw_data[1]) / 1000) + ' kHz'

            print(f' - codec: {acod}')
            print(f' - samplerate: {sr}')

            output = pipeToConsole([ffprobe, '-v', 'error', '-select_streams',
                'a:0', '-show_entries', 'stream=bit_rate', '-of',
                'compact=p=0:nk=1', file]).strip()
            if(output.isnumeric()):
                abit = str(round(int(output) / 1000)) + 'k'
                print(f' - bitrate: {abit}')
        else:
            print('Invalid media.')
    print('')



'''setup.py'''

"""
This code should only be executed by developers, not users.

The purose of this script is to create the build for pip.
"""

import os
import sys
from setuptools import setup, find_packages

def pip_version():
    # pip doesn't allow standard version format (20w10a), so we have to
    # conform it to match Semantic Versioning.
    return '21.1.1'

if(sys.argv[-1] == 'publish'):
    os.system('rm -rf build')
    os.system('rm -rf dist')
    os.system('python3 setup.py sdist bdist_wheel')
    os.system('twine upload dist/*')
    sys.exit()

with open('README.md', 'r') as f:
    long_description = f.read()

setup(
    name='auto-editor',
    version=pip_version(),
    description='Auto-Editor: Effort free video editing!',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='Unlicense',
    url='https://github.com/WyattBlue/auto-editor',
    author='WyattBlue',
    author_email='wyattbluesandbox@gmail.com',
    keywords='video audio media editor editing processing nonlinear automatic ' \
     'silence-detect silence-removal silence-speedup motion-detection',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'numpy==1.19.3',
        'audiotsm2',
        'opencv-python',
        'youtube-dl',
        'requests',
    ],
    classifiers=[
        'Topic :: Multimedia :: Video',
        'License :: Public Domain',
        'License :: OSI Approved :: The Unlicense (Unlicense)',
        'Environment :: Console',
        'Natural Language :: English',
        'Development Status :: 5 - Production/Stable',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Programming Language :: Python :: Implementation :: IronPython',
        'Programming Language :: Python :: Implementation :: Jython',
    ],
    entry_points={
        "console_scripts": ["auto-editor=auto_editor.__main__:main"]
    }
)


'''fastVideo.py'''

import numpy as np

# Included functions
from fastAudio import fastAudio
from usefulFunctions import ProgressBar, pipeToConsole, ffAddDebug

# Internal libraries
import subprocess
from shutil import move

def handleAudioTracks(ffmpeg, outFile, exportAsAudio, tracks, keepTracksSep,
    chunks, speeds, fps, temp, machineReadable, hideBar, log):
    import os

    log.checkType(tracks, 'tracks', int)
    log.checkType(outFile, 'outFile', str)

    for trackNum in range(tracks):
        fastAudio(f'{temp}/{trackNum}.wav', f'{temp}/new{trackNum}.wav', chunks,
            speeds, log, fps, machineReadable, hideBar)

        if(not os.path.isfile(f'{temp}/new{trackNum}.wav')):
            log.bug('Audio file not created.')

    if(exportAsAudio):
        if(keepTracksSep):
            log.warning("Audio files can't have multiple tracks.")

        if(tracks == 1):
            move(f'{temp}/0.wav', outFile)
            return None # Exit out early.

        cmd = [ffmpeg, '-y']
        for trackNum in range(tracks):
            cmd.extend(['-i', f'{temp}/{trackNum}.wav'])
        cmd.extend(['-filter_complex', f'amix=inputs={tracks}:duration=longest', outFile])
        log.debug(cmd)
        subprocess.call(cmd)
        return False
    return True

def muxVideo(ffmpeg, outFile, keepTracksSep, tracks, vbitrate, tune, preset, vcodec,
    acodec, crf, temp, log):

    def extender(cmd, vbitrate, tune, preset, acodec, outFile, isFFmpeg):
        if(acodec is not None):
            cmd.extend(['-c:a', acodec])
        if(vbitrate is None):
            cmd.extend(['-crf', crf])
        else:
            cmd.extend(['-b:v', vbitrate])
        if(tune != 'none'):
            cmd.extend(['-tune', tune])
        cmd.extend(['-preset', preset, '-movflags', '+faststart', '-strict', '-2',
            outFile])
        cmd = ffAddDebug(cmd, isFFmpeg)
        return cmd

    # Now mix new audio(s) and the new video.
    if(keepTracksSep):
        cmd = [ffmpeg, '-y']
        for i in range(tracks):
            cmd.extend(['-i', f'{temp}/new{i}.wav'])
        cmd.extend(['-i', f'{temp}/spedup.mp4'])
        for i in range(tracks):
            cmd.extend(['-map', f'{i}:a:0'])

        cmd.extend(['-map', f'{tracks}:v:0', '-c:v', vcodec])
        cmd = extender(cmd, vbitrate, tune, preset, acodec, outFile, log.is_ffmpeg)
    else:
        # Merge all the audio tracks into one.
        if(tracks > 1):
            cmd = [ffmpeg]
            for i in range(tracks):
                cmd.extend(['-i', f'{temp}/new{i}.wav'])
            cmd.extend(['-filter_complex', f'amerge=inputs={tracks}', '-ac', '2',
                f'{temp}/newAudioFile.wav'])
            if(log.is_ffmpeg):
                cmd.extend(['-hide_banner'])
            else:
                cmd.extend(['-nostats', '-loglevel', '8'])
            subprocess.call(cmd)
        else:
            move(f'{temp}/new0.wav', f'{temp}/newAudioFile.wav')

        cmd = [ffmpeg, '-y', '-i', f'{temp}/newAudioFile.wav', '-i',
            f'{temp}/spedup.mp4', '-c:v', vcodec]
        cmd = extender(cmd, vbitrate, tune, preset, acodec, outFile, log.is_ffmpeg)

    log.debug(cmd)
    message = pipeToConsole(cmd)
    log.debug(message)

    if('Conversion failed!' in message):
        log.warning('The muxing/compression failed. '\
            'This may be a problem with FFmpeg, your video codec, '\
            'or your video bitrate. \nTrying, again but not compressing.')
        cmd = [ffmpeg, '-y', '-i', f'{temp}/newAudioFile.wav', '-i',
            f'{temp}/spedup.mp4', '-c:v', 'copy', '-movflags', '+faststart',
            outFile]
        cmd = ffAddDebug(cmd, log.is_ffmpeg)
        subprocess.call(cmd)
    log.conwrite('')


def fastVideo(vidFile: str, chunks: list, includeFrame: np.ndarray, speeds: list,
    fps, machineReadable, hideBar, temp, log):

    import cv2

    cap = cv2.VideoCapture(vidFile)
    width = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))
    height = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
    fourcc = cv2.VideoWriter_fourcc(*'mp4v')

    out = cv2.VideoWriter(f'{temp}/spedup.mp4', fourcc, fps, (width, height))

    log.checkType(vidFile, 'vidFile', str)
    log.checkType(includeFrame, 'includeFrame', np.ndarray)

    if(speeds[0] == 99999 and speeds[1] != 99999):
        totalFrames = int(np.where(includeFrame == True)[0][-1])
        cframe = int(np.where(includeFrame == True)[0][0])
    elif(speeds[0] != 99999 and speeds[1] == 99999):
        totalFrames = int(np.where(includeFrame == False)[0][-1])
        cframe = int(np.where(includeFrame == False)[0][0])
    else:
        totalFrames = chunks[len(chunks) - 1][1]
        cframe = 0

    starting = cframe
    cap.set(cv2.CAP_PROP_POS_FRAMES, cframe)
    remander = 0
    framesWritten = 0

    videoProgress = ProgressBar(totalFrames - starting, 'Creating new video',
        machineReadable, hideBar)

    while cap.isOpened():
        ret, frame = cap.read()
        if(not ret or cframe > totalFrames):
            break

        cframe = int(cap.get(cv2.CAP_PROP_POS_FRAMES)) # current frame
        try:
            state = includeFrame[cframe]
        except IndexError:
            state = False

        mySpeed = speeds[state]

        if(mySpeed != 99999):
            doIt = (1 / mySpeed) + remander
            for __ in range(int(doIt)):
                out.write(frame)
                framesWritten += 1
            remander = doIt % 1

        videoProgress.tick(cframe - starting)
    log.debug(f'\n   - Frames Written: {framesWritten}')
    log.debug(f'   - Starting: {starting}')
    log.debug(f'   - Total Frames: {totalFrames}')

    if(log.is_debug):
        log.debug('Writing the output file.')
    else:
        log.conwrite('Writing the output file.')

    cap.release()
    out.release()
    cv2.destroyAllWindows()



'''mediaMetadata.py'''

from usefulFunctions import pipeToConsole
import re

def vidTracks(videoFile: str, ffprobe: str, log) -> int:
    """
    Return the number of audio tracks in a video file.
    """
    numbers = pipeToConsole([ffprobe, videoFile, '-hide_banner', '-loglevel',
        'panic', '-show_entries', 'stream=index', '-select_streams', 'a', '-of',
        'compact=p=0:nk=1']).split('\n')

    # Remove all blanks
    numbers = [s for s in numbers if s != '']
    # Remove all \r chars that can appear in certain environments
    numbers = [s.replace('\r', '') for s in numbers]

    log.ffmpeg('Track data: ' + str(numbers))
    if(numbers[0].isnumeric()):
        return len(numbers)
    else:
        log.warning('ffprobe had an invalid output.')
        return 1 # Assume there's one audio track.


def getVideoCodec(file: str, ffmpeg: str, log, vcodec: str) -> str:
    if(vcodec == 'copy'):
        output = pipeToConsole([ffmpeg, '-i', file, '-hide_banner'])
        try:
            matchDict = re.search(r'Video:\s(?P<grp>\w+?)\s', output).groupdict()
            vcodec = matchDict['grp']
            log.debug('Video Codec: ' + str(vcodec))
        except AttributeError:
            log.warning("Couldn't automatically detect video codec.")
    if(vcodec is None or vcodec == 'uncompressed'):
        vcodec = 'copy'
    return vcodec


def getSampleRate(file: str, ffmpeg: str, sr) -> str:
    if(sr is None):
        output = pipeToConsole([ffmpeg, '-i', file, '-hide_banner'])
        try:
            matchDict = re.search(r'\s(?P<grp>\w+?)\sHz', output).groupdict()
            return matchDict['grp']
        except AttributeError:
            return '48000'
    return str(sr)


def getAudioBitrate(file: str, ffprobe: str, log, abit: str) -> str:
    if(abit is None):
        if(file.endswith('.mkv')):
            return None
        else:
            output = pipeToConsole([ffprobe, '-v', 'error', '-select_streams',
                'a:0', '-show_entries', 'stream=bit_rate', '-of',
                'compact=p=0:nk=1', file]).strip()
            if(output.isnumeric()):
                return str(round(int(output) / 1000)) + 'k'
            else:
                log.warning("Couldn't automatically detect audio bitrate.")
                log.debug('Setting audio bitrate to 500k')
                log.debug(f'Output: {output}')
                return '500k'
    return str(abit)


def ffmpegFPS(ffmpeg: str, path: str, log) -> float:
    output = pipeToConsole([ffmpeg, '-i', path, '-hide_banner'])
    try:
        matchDict = re.search(r'\s(?P<fps>[\d\.]+?)\stbr', output).groupdict()
        return float(matchDict['fps'])
    except AttributeError:
        log.warning('frame rate detection failed.\n' \
            'If your video has a variable frame rate, ignore this message.')
        return 30.0
